/* Stats - IRC Statistical Services
** Original Copyright (c) 1999 Jonathan George
**  E-Mail: net@lite.net
*
** Stats CVS Identification
** $Id: conf.c,v 1.44 2002/01/14 22:47:46 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"

static void strip(char *);
static void AddChannel(char *chan);

struct ConfTable
{
	char *name;
	int rehashable;
	void (*handler)();
};

static void c_server_name(char *);
static void c_server_port(char *);
static void c_connect_to(char *);
static void c_connect_pass(char *);
static void c_use_vhost(char *);
static void c_vhost(char *);
static void c_server_desc(char *);
static void c_netname(char *);
static void c_admin_name(char *);
static void c_admin_email(char *);
static void c_reconnect_time(char *);
static void c_flood(char *);
static void c_flood_time(char *);
static void c_flood_ignore(char *);
static void c_lag_time(char *);
static void c_hide_server(char *);
static void c_statserv_user(char *);
static void c_statserv_host(char *);
static void c_statserv_nick(char *);
static void c_enable_spam(char *);
static void c_onlyopers(char *);
static void c_want_privmsg(char *);
static void c_lag_notices(char *);
static void c_enable_html(char *);
static void c_set_oper(char *);
static void c_html_dump(char *);
static void c_html_time(char *);
static void c_ircd_type(char *);
static void c_channels(char *);
static void c_url(char *);
static void c_splitserv_desc(char *);
static void c_splitserv_delay(char *);
static void c_splitserv_host(char *);
static void c_splitserv_user(char *);
static void c_splitserv_nick(char *);
static void c_splitserv_host(char *);
static void c_splitserv_privmsg(char *);
static void c_splitserv_chanmsg(char *);
static void c_splitserv_enable_text(char *);
static void c_splitserv_enable_expire(char *);
static void c_splitserv_expire_time(char *);
static void c_splitserv_report(char *);
static void c_splitserv_text_time(char *);
static void c_splitserv_text_name(char *);
static void c_splitserv_chan(char *);
static void c_adminserv_host(char *);
static void c_adminserv_nick(char *);
static void c_adminserv_user(char *);
static void c_jupeserv_host(char *);
static void c_jupeserv_nick(char *);
static void c_jupeserv_user(char *);

static struct ConfTable conf_table[] =
{
	{ "HIDE_SERVER",	1,	c_hide_server },
	{ "SERVER_NAME",	0,	c_server_name },
	{ "SERVER_PORT",	0,	c_server_port },
	{ "CONNECT_TO",		0,	c_connect_to },
	{ "CONNECT_PASS",	0,	c_connect_pass },
	{ "USE_VHOST",		0,	c_use_vhost },
	{ "VHOST",		0,	c_vhost },
	{ "SERVER_DESC",	0,	c_server_desc },
	{ "NETNAME",		1,	c_netname },
	{ "ADMIN_NAME",		1,	c_admin_name },
	{ "ADMIN_EMAIL",	1,	c_admin_email },
	{ "RECONNECT_TIME",	1,	c_reconnect_time },
	{ "FLOOD_TIME",		1,	c_flood_time },
	{ "FLOOD",		1,	c_flood },
	{ "FLOOD_IGNORE",	1,	c_flood_ignore },
	{ "LAG_TIME",		1,	c_lag_time },
	{ "STATSERV_HOST",	0,	c_statserv_host },
	{ "STATSERV_USER",	0,	c_statserv_user },
	{ "STATSERV_NICK",	0,	c_statserv_nick },
	{ "ENABLE_SPAM",	0,	c_enable_spam },
	{ "ONLYOPERS",		1,	c_onlyopers },
	{ "WANT_PRIVMSG",	1,	c_want_privmsg },
	{ "LAG_NOTICES",	1,	c_lag_notices },
	{ "ENABLE_HTML",	1,	c_enable_html },
	{ "SET_OPER",		0,	c_set_oper },
	{ "HTML_DUMP",		1,	c_html_dump },
	{ "HTML_TIME",		1,	c_html_time },
	{ "IRCD_TYPE",		0,	c_ircd_type },
	{ "CHANNELS",		0,	c_channels },
	{ "URL",		1,	c_url },
	{ "SPLITSERV_DESC",	0,	c_splitserv_desc },
	{ "SPLITSERV_DELAY",	1,	c_splitserv_delay },
	{ "SPLITSERV_HOST",	0,	c_splitserv_host },
	{ "SPLITSERV_USER",	0,	c_splitserv_user },
	{ "SPLITSERV_NICK",	0,	c_splitserv_nick },
	{ "SPLITSERV_PRIVMSG",	1,	c_splitserv_privmsg },
	{ "SPLITSERV_CHANMSG",	1,	c_splitserv_chanmsg },
	{ "SPLITSERV_ENABLE_TEXT",	1,	c_splitserv_enable_text },
	{ "SPLITSERV_ENABLE_EXPIRE",	1,	c_splitserv_enable_expire },
	{ "SPLITSERV_EXPIRE_TIME",	1,	c_splitserv_expire_time },
	{ "SPLITSERV_REPORT",	1,	c_splitserv_report },
	{ "SPLITSERV_TEXT_TIME",1,	c_splitserv_text_time },
	{ "SPLITSERV_TEXT_NAME",1,	c_splitserv_text_name },
	{ "SPLITSERV_CHAN",	0,	c_splitserv_chan },
	{ "ADMINSERV_USER",	0,	c_adminserv_user },
	{ "ADMINSERV_HOST",	0,	c_adminserv_host },
	{ "ADMINSERV_NICK",	0,	c_adminserv_nick },
	{ "JUPESERV_HOST",	0,	c_jupeserv_host },
	{ "JUPESERV_USER",	0,	c_jupeserv_user },
	{ "JUPESERV_NICK",	0,	c_jupeserv_nick },
	{ (char *) 0,		0,	(void (*)()) 0 }
};

/* Remove CR and LF from the line input.  ASSUME that the line 
** does NOT contain CR or LF in the middle
*/
static void strip(char *line)
{
	char *c;

	if ((c = strchr(line, '\r')))
	{
		/* If we receive a CR, then we can just terminate the
		** call there.
		*/
		*c = '\0';
		return;
	}

	if ((c = strchr(line, '\n')))
	{
		*c = '\0';
		return;
	}

	return;
}

/* Set default parameters.  The settings used are the same as
** what is provided in stats.cfg.dist except for the binary
** ones, which MUST be default 0 in order for the conf load
** to work properly
*/
void InitDefaults()
{
	/* These strcpy's are safe, as they are static strings */

	/* All bit values must be initialized to 0 */
	strcpy(settings.name, "stats.network.tld");
	settings.port = 6666;
	strcpy(settings.uplink, "hub.network.tld");
	strcpy(settings.vhost, "0.0.0.0");
	strcpy(settings.pass, "password");
	strcpy(settings.desc, "Network Statistical Services");
	settings.flags=0x00000000; /* 0 is fine, but be explicit */
	settings.r_time = 10;
	settings.flood = 6;
	settings.flood_time = 10;
#ifdef STATSERV
	strcpy(StatServ.host, "stats.network.tld");
	strcpy(StatServ.user, "stats");
	strcpy(StatServ.nick, "StatServ");
#endif
	settings.lag_time = 45;
	strcpy(settings.html, "/home/user/public_html/stats.html");
	settings.html_time = 3600;
	strcpy(settings.join_chans, "#statserv,#opers");
	strcpy(settings.url, "http://www.network.tld/stats.html");
	strcpy(settings.netname, "Misconfigured");
	strcpy(settings.admin_name, "Misconfigured Admin (mis-nick)");
	strcpy(settings.admin_email, "admin@misconfigured.net");
	/* The chan_list is derived from the join_chans...don't set it */
#ifdef SPLITSERV
	strcpy(sp.desc, "Split Services");
	sp.delay = 15;
	strcpy(SplitServ.host, "split.network.tld");
	strcpy(SplitServ.user, "split");
	strcpy(SplitServ.nick, "SplitServ");
	sp.flags = 0;
	sp.text_time = 1800;
	sp.expire_time = 172800L;
	strcpy(sp.text_name, "map.txt");
	strcpy(sp.join_chan, "#split");
#endif
	strcpy(AdminServ.nick, "AdminServ");
	strcpy(AdminServ.user, "admin");
	strcpy(AdminServ.host, "admin.network.tld");
#ifdef JUPESERV
	strcpy(JupeServ.nick, "JupeServ");
	strcpy(JupeServ.user, "jupe");
	strcpy(JupeServ.host, "jupe.network.tld");
#endif
}

/* Used to verify setting from the conf during startup and rehash...
** This prevents invalid variables from being used
*/
void VerifyConf()
{
        /* Sanity checking for FLOOD */
        if (settings.flood == 1)
        {
                settings.flood = 2;
        }
}

void ConfLoad()
{
        FILE *fp;
        char buf[BUFSIZE];
	char *cmd = buf;
        char *param;
	int i, found;

        if ((fp = fopen(CONFIG_NAME, "r")) == NULL)
        {
                printf("Unable to open configuration file (%s)\n", CONFIG_NAME);
                exit(0);
        }

        while (fgets(buf, BUFSIZE, fp))
	{
		strip(buf);
		if (!buf[0])
			continue;
                if (buf[0] == '#')
                        continue;
		param = strchr(buf, ' ');
		if (param)
		{
			*param = '\0';
			param++;
		}
		else
		{
			param = NULL;
		}
		found = 0;
		
		for (i=0; conf_table[i].name; i++)
		{
			/* using strncasecmp will be bad here, consider
			** "FLOOD"
			*/
			if (!strcasecmp(cmd, conf_table[i].name))
			{
				conf_table[i].handler(param);
				found = 1;
				break;
			}
		}

		if (!found)
			log("Invalid conf file option: %s", cmd);
	}

	fclose(fp);
}

static void c_server_name(char *param)
{
	if (param)
	{
		memcpy(settings.name, param, sizeof(settings.name));
		settings.name[MAXHOST - 1] = '\0';
	}
	else
	{
		log("Invalid parameter for SERVER_NAME");
	}
}

static void c_hide_server(char *param)
{
	if (param)
	{
		AddHidden(param, 1);
	}
	else
	{
		log("Invalid parameter for HIDE_SERVER");
	}
}

static void c_server_port(char *param)
{
        if (param)
        {
		settings.port = atoi(param);
        }
        else
        {
                log("Invalid parameter for SERVER_PORT");
        }
}

static void c_connect_to(char *param)
{
        if (param)
	{
                memcpy(settings.uplink, param, sizeof(settings.uplink));
                settings.uplink[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for CONNECT_TO");
        }
}

static void c_connect_pass(char *param)
{
        if (param)
	{
                memcpy(settings.pass, param, sizeof(settings.pass));
                settings.pass[MAXPASS - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for CONNECT_PASS");
        }
}

static void c_use_vhost(char *param)
{
	settings.flags |= FLAG_VHOST;
}

static void c_vhost(char *param)
{
        if (param)
	{
                memcpy(settings.vhost, param, sizeof(settings.vhost));
                settings.vhost[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for VHOST");
        }
}

static void c_server_desc(char *param)
{
        if (param)
	{
                memcpy(settings.desc, param, sizeof(settings.desc));
                settings.desc[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SERVER_DESC");
        }
}

static void c_netname(char *param)
{
        if (param)
	{
                memcpy(settings.netname, param, sizeof(settings.netname));
                settings.netname[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for NETNAME");
        }
}

static void c_admin_name(char *param)
{
        if (param)
	{
                memcpy(settings.admin_name, param, sizeof(settings.admin_name));
                settings.admin_name[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMIN_NAME");
        }
}

static void c_admin_email(char *param)
{
        if (param)
	{
                memcpy(settings.admin_email, param,
			sizeof(settings.admin_email));
                settings.admin_email[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMIN_EMAIL");
        }
}

static void c_reconnect_time(char *param)
{
        if (param)
	{
		settings.r_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for RECONNECT_TIME");
        }
}

static void c_flood(char *param)
{
        if (param)
	{
		settings.flood = atoi(param);
        }
        else
        {
                log("Invalid parameter for FLOOD");
        }
}

static void c_flood_time(char *param)
{
        if (param)
	{
		settings.flood_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for FLOOD_TIME");
        }
}

static void c_flood_ignore(char *param)
{
	settings.flags |= FLAG_FLOOD;
}

static void c_lag_time(char *param)
{
        if (param)
	{
		settings.lag_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for LAG_TIME");
        }
}

static void c_statserv_host(char *param)
{
#ifdef STATSERV
        if (param)
	{
                memcpy(StatServ.host, param, sizeof(StatServ.host));
                StatServ.host[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for STATSERV_HOST");
        }
#endif
}

static void c_statserv_user(char *param)
{
#ifdef STATSERV
        if (param)
	{
                memcpy(StatServ.user, param, sizeof(StatServ.user));
                StatServ.user[MAXUSER - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for STATSERV_USER");
        }
#endif
}

static void c_statserv_nick(char *param)
{
#ifdef STATSERV
        if (param)
	{
                memcpy(StatServ.nick, param, sizeof(StatServ.nick));
                StatServ.nick[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for STATSERV_NICK");
        }
#endif
}

static void c_enable_spam(char *param)
{
	printf("ENABLE_SPAM is a compile time option.\n");
	printf("Please remove this option from the\n");
	printf("%s file.\nNOT FATAL\n", CONFIG_NAME);
}

static void c_onlyopers(char *param)
{
	settings.flags |= FLAG_ONLYOPERS;
}

static void c_want_privmsg(char *param)
{
	settings.flags |= FLAG_PRIVMSG;
}

static void c_lag_notices(char *param)
{
	settings.flags |= FLAG_LAG;
}

static void c_enable_html(char *param)
{
	settings.flags |= FLAG_HTML;
}

static void c_set_oper(char *param)
{
	settings.flags |= FLAG_SETOPER;
}

static void c_html_dump(char *param)
{
        if (param)
        {
                memcpy(settings.html, param, sizeof(settings.html));
                settings.html[BUFSIZE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for HTML_DUMP");
        }
}

static void c_html_time(char *param)
{
        if (param)
        {
		settings.html_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for HTML_TIME");
        }
}

static void c_ircd_type(char *param)
{
	printf("The IRCD_TYPE Option has been disabled.\n");
	printf("Please remove the IRCD_TYPE from %s\n", CONFIG_NAME);
	printf("and choose the IRCD_type define in config.h\n");
	printf("\nStats terminated.\n");
	exit(1);
}

static void c_channels(char *param)
{
        if (param)
        {
		AddChannel(param);
        }
        else
        {
                log("Invalid parameter for CHANNELS");
        }
}

static void c_url(char *param)
{
        if (param)
        {
                memcpy(settings.url, param, sizeof(settings.url));
                settings.url[BUFSIZE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for URL");
        }
}

static void c_splitserv_desc(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                memcpy(sp.desc, param, sizeof(sp.desc));
                sp.desc[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_DESC");
        }
#endif
}

static void c_splitserv_delay(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
		sp.delay = atoi(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_DELAY");
        }
#endif
}

static void c_splitserv_host(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                memcpy(SplitServ.host, param, sizeof(SplitServ.host));
                SplitServ.host[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_HOST");
        }
#endif
}

static void c_splitserv_user(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                memcpy(SplitServ.user, param, sizeof(SplitServ.user));
                SplitServ.user[MAXUSER - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_USER");
        }
#endif
}

static void c_splitserv_nick(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                memcpy(SplitServ.nick, param, sizeof(SplitServ.nick));
                SplitServ.nick[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_NICK");
        }
#endif
}

static void c_splitserv_privmsg(char *param)
{
#ifdef SPLITSERV
	sp.flags |= SPF_PRIVMSG;
#endif
}

static void c_splitserv_chanmsg(char *param)
{
#ifdef SPLITSERV
	sp.flags |= SPF_CHANMSG;
#endif
}

static void c_splitserv_enable_text(char *param)
{
#ifdef SPLITSERV
        sp.flags |= SPF_ENABLE_TEXT;
#endif
}

static void c_splitserv_enable_expire(char *param)
{
#ifdef SPLITSERV
	sp.flags |= SPF_EXPIRE;
#endif
}

static void c_splitserv_report(char *param)
{
#ifdef SPLITSERV
        sp.flags |= SPF_REPORT;
#endif
}

static void c_splitserv_text_time(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
		sp.text_time = atoi(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_TEXT_TIME");
        }
#endif
}

static void c_splitserv_expire_time(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                sp.expire_time = atol(param);
        }
        else
        {
                log("Invalid parameter for SPLITSERV_EXPIRE_TIME");
        }
#endif
}

static void c_splitserv_text_name(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                memcpy(sp.text_name, param, sizeof(sp.text_name));
                sp.text_name[BUFSIZE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_TEXT_NAME");
        }
#endif
}

static void c_splitserv_chan(char *param)
{
#ifdef SPLITSERV
        if (param)
        {
                memcpy(sp.join_chan, param, sizeof(sp.join_chan));
                sp.join_chan[LINE - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for SPLITSERV_CHAN");
        }
#endif
}

static void c_adminserv_user(char *param)
{
        if (param)
        {
                memcpy(AdminServ.user, param, sizeof(AdminServ.user));
                AdminServ.user[MAXUSER - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMINSERV_USER");
        }
}

static void c_adminserv_host(char *param)
{
        if (param)
        {
                memcpy(AdminServ.host, param, sizeof(AdminServ.host));
                AdminServ.host[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMINSERV_HOST");
        }
}

static void c_adminserv_nick(char *param)
{
        if (param)
        {
                memcpy(AdminServ.nick, param, sizeof(AdminServ.nick));
                AdminServ.nick[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for ADMINSERV_NICK");
        }
}

static void c_jupeserv_user(char *param)
{
#ifdef JUPESERV
	if (param)
	{
		memcpy(JupeServ.user, param, sizeof(JupeServ.user));
		JupeServ.user[MAXUSER - 1] = '\0';
	}
	else
	{
		log("Invalid parameter for JUPESERV_USER");
	}
#endif
}

static void c_jupeserv_host(char *param)
{
#ifdef JUPESERV
        if (param)
        {
                memcpy(JupeServ.host, param, sizeof(JupeServ.host));
                JupeServ.host[MAXHOST - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for JUPESERV_HOST");
        }
#endif
}

static void c_jupeserv_nick(char *param)
{
#ifdef JUPESERV
        if (param)
        {
                memcpy(JupeServ.nick, param, sizeof(JupeServ.nick));
                JupeServ.nick[MAXNICK - 1] = '\0';
        }
        else
        {
                log("Invalid parameter for JUPESERV_NICK");
        }
#endif
}

static void AddChannel(char *chan)
{
	char *s = NULL;
	char *pos = chan;
	while ( (s = strchr(pos, ',')) && (netinfo.chan_count < MAXCHANS) )
	{
		*s = '\0';
		s++;
		if (*s == ' ')
			s++;
		me.chan_list[netinfo.chan_count] = strdup(pos);
		netinfo.chan_count++;
		pos = s;
	}

	if (pos && (*pos != '\n') && (netinfo.chan_count < MAXCHANS) )
	{
		me.chan_list[netinfo.chan_count] = strdup(pos);
		netinfo.chan_count++;
	}
}

void rehash()
{
	/* Should I free and reload the TLD file here? */

        FILE *fp;
        char buf[BUFSIZE];
        char *cmd = buf;
        char *param;
        int i, found;

	/* First things first...reset the bit flags to 0 */
	settings.flags = 0;
#ifdef SPLITSERV
	sp.flags = 0;
#endif

	/* Flush the hidden servers list */
	while (hiddenhead.head)
	{
		Hidden *h = hiddenhead.head->data;
		dlink_node *dl = hiddenhead.head;
		dlinkDelete(hiddenhead.head, &hiddenhead);
		free(h);
		free(dl);
	}

#if 0
	/* Woah...should not do this in a REHASH */

	/* Flush the channel list */
	for (i = 0; i < MAXCHANS; i++)
	{
		if (me.chan_list[i])
		{
			free(me.chan_list[i]);
			me.chan_list[i] = 0;
		}
		netinfo.chan_count = 0;
	}
#endif

	/* Weee, done the preliminary stuff */

        if ((fp = fopen(CONFIG_NAME, "r")) == NULL)
        {
                printf("Unable to open configuration file (%s)\n", CONFIG_NAME);
                exit(0);
        }

        while (fgets(buf, BUFSIZE, fp))
        {
                strip(buf);
                if (!buf[0])
                        continue;
                if (buf[0] == '#')
                        continue;
                param = strchr(buf, ' ');
                if (param)
                {
                        *param = '\0';
                        param++;
                }
                else
                {
                        param = NULL;
                }
                found = 0;

                for (i=0; conf_table[i].name; i++)
                {
                        /* using strncasecmp will be bad here, consider
                        ** "FLOOD"
                        */
                        if (!strcasecmp(cmd, conf_table[i].name) &&
				conf_table[i].rehashable )
                        {
                                conf_table[i].handler(param);
                                found = 1;
                                break;
                        }
                }

                if (!found)
                        log("Invalid conf file option: %s", cmd);
        }

        fclose(fp);

	VerifyConf();
}

