/* Stats - jupe.c - JupeServ backend functions
** W. Campbell (21 octobre 2001)
** $Id: jupe.c,v 1.16 2002/02/23 14:45:47 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "patchlevel.h"

#ifdef JUPESERV

void LoadJupes()
{
        FILE *fp;
        int parc;
        int i;
        char *parv[MAXPARA];
        char buf[BUFSIZE];

        /* Clear the parv */
        for (i = 0; i < MAXPARA; i++)
                parv[i] = NULL;

        if ((fp = fopen(JUPEFILE, "r")) == NULL)
        {
                log("Unable to load the jupe configuration file.");
                return;
        }

        while (fgets(buf, BUFSIZE, fp))
        {
                parc = msgtoken(buf, parv);

                if ((!parv[0]) || (!parv[1]) || (!parv[2]))
                {
                        log("Invalid entry in jupe configuration file");
                        log("Ignoring...please correct");
                        continue;
                }

                AddJupe(parv[0], parv[1], atol(parv[2]));
        }
        fclose(fp);
}

void SaveJupes()
{
        Jupe *j;
	dlink_node *dl;
        FILE *fp;

        /* I should zero out the jupe config file here */
        if (!jupehead.head)
	{
		truncate(JUPEFILE, 0);
                return;
	}

        if ((fp = fopen(JUPEFILE, "w")) == NULL)
        {
                log("Unable to open jupe configuration file for writing");
                return;
        }

        for (dl = jupehead.head; dl; dl = dl->next)
        {
		j = (Jupe *) dl->data;
                fprintf(fp, "%s %s %ld\n", j->jupe, j->oper, j->ts);
        }
        fclose(fp);
}

void AddJupe(char *jupe, char *oper, long ts)
{
        Jupe *j = smalloc(sizeof(Jupe));
        Jupe *check = FindJupe(jupe);
	dlink_node *dl = smalloc(sizeof(dlink_node));

        /* Don't add a jupe if it's already there */
        if (check)
	{
		/* but only after you free what you allocated... */
		free(j);
		free(dl);
                return;
	}

        strncpy(j->jupe, jupe, MAXHOST);
        strncpy(j->oper, oper, MAXNICK);
        j->jupe[MAXHOST] = '\0';
        j->oper[MAXNICK] = '\0';
        j->ts = ts;

	dlinkAdd(j, dl, &jupehead);

        /* Added the data structure Jupe record, now force it off the
        ** network and introduce our fake server in it's place
        */
	InitJupe(j);
}

void InitJupe(Jupe *j)
{
	Server *s = findserver(j->jupe);
        char jreason[LINE + 1];
#ifdef SPLITSERV
        time_t now = time(NULL);
        SpServer *spl = NULL;
        char *td = NULL;
#endif

        if (s)
                sts(":%s SQUIT %s :JUPED by %s", JupeServ.nick, j->jupe,
			j->oper);

	snprintf(jreason, LINE, "[Jupitered] by %s", j->oper);
        sts(":%s SERVER %s 1 :[Jupitered] by %s", settings.name, j->jupe,
		j->oper);

	AddServer(j->jupe, 1, jreason, settings.name);
#ifdef SPLITSERV
	spl = SpFindSplit(j->jupe);
	if (spl)
	{
		td = timediff(now - spl->ts);
		splog("End of Split: %s to %s: %s", j->jupe, settings.name,
			td);
		if (IsSPReport())
			privmsg(sp.join_chan, SplitServ.nick, MSG_SPDEFAULT,
				"END OF SPLIT: %s to %s: %s", j->jupe,
				settings.name, td);
	}
	else
	{
		if (!is_delayed)
		{
			splog("New Server: %s to %s", j->jupe, settings.name);
			if (IsSPReport())
				privmsg(sp.join_chan, SplitServ.nick,
				MSG_SPDEFAULT, "CONNECT: %s to %s", j->jupe,
				settings.name);
		}
	}
	SpNewServer(j->jupe, settings.name);
	SpDelSplit(j->jupe);
#endif
}

void DelJupe(char *jupe)
{
        Jupe *j;
	dlink_node *dl = jupehead.head;

        while (dl)
        {
		j = (Jupe *) dl->data;
                if (!strcasecmp(j->jupe, jupe))
                {
			dlinkDelete(dl, &jupehead);
                        free(j);
			free(dl);

                        /* Remove the juped server from the network */
                        sts("SQUIT %s :Unjuped", jupe);

                        return;
                }
                else
                {
                        dl = dl->next;
                }
        }

        log("Unable to delete jupe %s", jupe);
}

Jupe* FindJupe(char *jupe)
{
        Jupe *j;
	dlink_node *dl = jupehead.head;

        if (IsDebug())
                log("FindJupe(%s)", jupe);

        while (dl)
        {
		j = (Jupe *) dl->data;
                if (!strcasecmp(jupe, j->jupe))
                {
                        return j;
                }
                else
                {
                        dl = dl->next;
                }
        }

        return NULL;
}

void FlushJupeList()
{
	dlink_node *dl = jupehead.head;
	Jupe *j;

        while (jupehead.head)
        {
		dl = jupehead.head;
		j = (Jupe *) dl->data;
		dlinkDelete(dl, &jupehead);
		free(j);
                free(dl);
        }
}

#endif
